/* fcntl.c version 1.00 */

/****************
I want to provide POSIX-style fcntl locking in a non-posix application.
I will do this by replacing posix fcntl calls with the following
fcntl_to_flock, a lame fcntl lock emulator that uses flock

To compile this file for use in samba:
cc -c fcntl.c

****************/

/* All this jazz is not required if POSIX-fcntl is available */
#ifndef _POSIX_SOURCE

#if defined(fcntl)
#undef fcntl
#endif

#include <libc.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/file.h>
#include <sys/errno.h>

#ifndef __STRICT_BSD__ 
extern int errno;
#endif

/***** 
   When I lock exists, who to claim owns it.  The flock API doesn't
   support this, so this is simply BS.
   Samba doesn't like 0, so let's use the Devil's process #666. (-;
*****/ 
#define LOCK_OWNER 666 

int fcntl_to_flock(int fd, int cmd, struct flock *arg)
{
  int ret = 0;
  switch (cmd) {
    case F_SETLKW:	/* Blocking Lock */
      if ( (arg->l_type) == F_RDLCK )
        ret=flock(fd, LOCK_SH); 
      else if ( (arg->l_type) == F_WRLCK)
        ret = flock(fd, LOCK_EX); 
      else if (arg->l_type == F_UNLCK )
        ret = flock(fd, LOCK_UN);
      break;

    case F_SETLK:	 /* Nonblocking lock */
      if (arg->l_type == F_RDLCK)
        ret = flock(fd, LOCK_SH | LOCK_NB );
      else if (arg->l_type == F_WRLCK)
        ret = flock(fd, LOCK_EX | LOCK_NB );
      else if ( arg->l_type == F_UNLCK );
        ret = flock(fd, LOCK_UN | LOCK_NB);
      break; 

    case F_GETLK:	/* Get lock status... hmm... */
      if (arg->l_type == F_RDLCK)
      {
        ret = flock(fd, LOCK_SH | LOCK_NB );
        if ( ret == -1 && errno == EWOULDBLOCK)
	{
          arg->l_pid = LOCK_OWNER ; 
        }
        else 
        {  /* Undo the damage */
          flock(fd, LOCK_UN | LOCK_NB );
          arg->l_type = F_UNLCK;
        }
      }
      else if  (arg->l_type == F_WRLCK)
      {
        ret = flock(fd, LOCK_EX | LOCK_NB );
        if (ret == -1 && errno == EWOULDBLOCK)
        {  arg->l_pid = LOCK_OWNER ; }
        else 
        {  /* Undo the damage */
          flock(fd, LOCK_EX | LOCK_NB );
          arg->l_type = F_UNLCK;
        }
      }

      break;

    default:
	/* Other non-locking cmds get passed to the REAL (non-posix) fcntl */
	ret = fcntl(fd,cmd,(int) arg);
      break;

  } 

  return(ret);  

}
#endif /* _POSIX_SOURCE */
